"""
See PEP 386 (https://peps.python.org/pep-0386/)
"""
__version__ = "0.3.2"
__version_info__ = tuple(int(i) if i.isdigit() else i for i in __version__.split("."))
__license__ = "MIT"
__title__ = "model_mixin"

__author__ = "DLRSP"
__copyright__ = "Copyright 2010-present DLRSP"

# Version synonym
VERSION = __version_info__

# Header encoding (see RFC5987)
HTTP_HEADER_ENCODING = "iso-8859-1"

# Default datetime input and output formats
ISO_8601 = "iso-8601"



from django.contrib import admin, messages
from django.contrib.auth import get_permission_codename
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext


# TODO: check if update can be managed by save_model or clear cache key directly
class ModelAdminPublishFieldsMixin(admin.ModelAdmin):
    """Custom model mixin for Publish Date and Publish By information"""

    actions = ["make_published", "make_unpublished"]

    @admin.action(
        permissions=["publish"], description=_("Mark selected object as published")
    )
    def make_published(self, request, queryset):
        """Action to update selected object as published"""
        updated = queryset.update(published=True)
        self.message_user(
            request,
            ngettext(
                "%d object was successfully marked as published.",
                "%d objects were successfully marked as published.",
                updated,
            )
            % updated,
            messages.SUCCESS,
        )

    def has_publish_permission(self, request):
        """Does the user have the publish permission?"""
        opts = self.opts
        codename = get_permission_codename("publish", opts)
        return request.user.has_perm(f"{opts.app_label}.{codename}")

    @admin.action(
        permissions=["unpublish"], description=_("Mark selected object as unpublished")
    )
    def make_unpublished(self, request, queryset):
        """Action to update selected object as unpublished"""
        updated = queryset.update(published=False)
        self.message_user(
            request,
            ngettext(
                "%d object was successfully marked as unpublished.",
                "%d objects were successfully marked as unpublished.",
                updated,
            )
            % updated,
            messages.SUCCESS,
        )

    def has_unpublish_permission(self, request):
        """Does the user have the unpublish permission?"""
        opts = self.opts
        codename = get_permission_codename("unpublish", opts)
        return request.user.has_perm(f"{opts.app_label}.{codename}")

    def save_model(self, request, obj, form, change) -> None:
        """Update publish fields from request object before save."""
        current_now = timezone.now()
        if obj.published and not obj.publish:
            obj.publish = current_now
            obj.publish_by = request.user
        if not obj.published and obj.publish:
            obj.publish = None
            obj.publish_by = None
        super().save_model(request, obj, form, change)


class ModelAdminAuditFieldsMixin(admin.ModelAdmin):
    """Custom model mixin for Create/Modify Date and Create/Modify By information"""

    def save_model(self, request, obj, form, change) -> None:
        """Update audit fields from request object before save."""
        current_now = timezone.now()
        if not change:
            obj.created = current_now
            obj.created_by = request.user
        else:
            obj.modified = current_now
            obj.modified_by = request.user
        super().save_model(request, obj, form, change)



"""Test's settings"""
import os

from django.utils.translation import gettext_noop

DEBUG = False

BASE_DIR = os.path.dirname(os.path.abspath(__file__))

PASSWORD_HASHERS = ["django.contrib.auth.hashers.MD5PasswordHasher"]

SECRET_KEY = "NOTASECRET"

ALLOWED_HOSTS = ["*"]

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "model_mixin",
]

DATABASES = {"default": {"ENGINE": "django.db.backends.sqlite3"}}

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "APP_DIRS": True,
        "DIRS": [os.path.join(BASE_DIR, "templates")],
    }
]

ROOT_URLCONF = "tests.urls"

MIDDLEWARE = (
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.cache.UpdateCacheMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.middleware.locale.LocaleMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "django.middleware.cache.FetchFromCacheMiddleware",
)

USE_TZ = True
LANGUAGE_CODE = "en"
USE_I18N = True

STATIC_URL = "/static/"

# Languages we provide translations for, out of the box.
LANGUAGES = [
    ("de", gettext_noop("German")),
    ("en", gettext_noop("English")),
    ("es", gettext_noop("Spanish")),
    ("fr", gettext_noop("French")),
    ("it", gettext_noop("Italian")),
    ("ja", gettext_noop("Japanese")),
    ("nl", gettext_noop("Dutch")),
    ("ru", gettext_noop("Russian")),
    ("zh-hans", gettext_noop("Simplified Chinese")),
    ("zh-hant", gettext_noop("Traditional Chinese")),
]



from django.contrib.auth import get_user_model
from django.db import models
from django.utils.translation import gettext_lazy as _


class AuditModelMixin(models.Model):
    """
    Mixin that provides fields created and modified at and by fields
    """

    UserModel = get_user_model()
    created = models.DateTimeField(_("Created"), auto_now_add=True)
    created_by = models.ForeignKey(
        UserModel,
        verbose_name=_("Created By"),
        on_delete=models.CASCADE,
        related_name="%(class)s_created_by",
    )
    modified = models.DateTimeField(_("Modified"), blank=True, null=True)
    modified_by = models.ForeignKey(
        UserModel,
        blank=True,
        null=True,
        verbose_name=_("Modified By"),
        on_delete=models.CASCADE,
        related_name="%(class)s_modified_by",
    )

    class Meta:
        abstract = True


class PublishModelMixin(models.Model):
    """
    Mixin that provides fields publish at and by fields
    """

    UserModel = get_user_model()
    publish = models.DateTimeField(_("Published"), blank=True, null=True)
    publish_by = models.ForeignKey(
        UserModel,
        blank=True,
        null=True,
        verbose_name=_("Published By"),
        on_delete=models.CASCADE,
        related_name="%(class)s_publish_by",
    )
    published = models.BooleanField(_("Publish"), default=False)

    class Meta:
        abstract = True



"""Unit Tests for the module"""
import logging

from django.test import TestCase

LOGGER = logging.getLogger(name="django-model-mixin")


class TestCase(TestCase):
    """Test Case for django-model-mixin"""

    def setUp(self):
        """Set up common assets for tests"""
        LOGGER.debug("Tests setUp")

    def tearDown(self):
        """Remove Test Data"""
        LOGGER.debug("Tests tearDown")

    def test_fake(self):
        """Test return passed"""
        LOGGER.debug("Fake Test")
        self.assertEqual(0, 0)



"""Test's urls view for django-model-mixin"""
from django.conf.urls.i18n import i18n_patterns
from django.urls import include, path, re_path
from django.views.i18n import JavaScriptCatalog

urlpatterns = [
    path("", include("model_mixin.urls")),
]

urlpatterns += i18n_patterns(
    re_path(r"^jsi18n/$", JavaScriptCatalog.as_view(), name="javascript-catalog"),
    path("", include("model_mixin.urls")),
)



import os
from distutils.cmd import Command
from distutils.command.build import build as _build

from setuptools.command.install_lib import install_lib as _install_lib

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


class CompileTranslations(Command):
    description = "compile message catalogs to MO files via django compilemessages"
    user_options = []  # type: list

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        curdir = os.getcwd()
        os.chdir(
            os.path.realpath(
                os.path.join("src", os.path.basename(list(os.walk("src"))[1][0]))
            )
        )
        from django.core.management import call_command

        call_command("compilemessages")
        os.chdir(curdir)


class Build(_build):
    sub_commands = [] + _build.sub_commands
    if os.path.isdir(
        os.path.realpath(
            os.path.join("src", os.path.basename(list(os.walk("src"))[1][0]), "locale")
        )
    ):
        sub_commands = [("compile_translations", None)] + _build.sub_commands


class InstallLib(_install_lib):
    def run(self):
        _install_lib.run(self)


setup(
    cmdclass={
        "build": Build,
        "install_lib": InstallLib,
        "compile_translations": CompileTranslations,
    },
)
